#include <windows.h>
#define SG633USB_Exports
#include "sg633usb.h"
#include "Ftdio.h"
#include "Wake.h"

typedef char SStr[32];

//---------------------------- Constants: -----------------------------------

//Wake protocol constantes:

#define FRAME     200 //max frame size

//Wake protocol errors:

#define ERR_NO   0    //no error
#define ERR_TX   1    //IO error
#define ERR_BU   2    //device busy error
#define ERR_RE   3    //device not ready
#define ERR_PA   4    //parameters value error
#define ERR_NR   5    //device not responding

//Unknown error:

#define ERR_UK   6    //unknown error

#define MAX_ERR  6

//Errors names:

const SStr ErrorMsg[MAX_ERR + 1] =
  { "",
    "invalid packet.",
    "device busy.",
    "device not ready.",
    "invalid parameters.",
    "device not responding.",
    "unknown." };

//Wake protocol commands:

#define CMD_NOP        0 //no operation
#define CMD_ERR        1 //get error code
#define CMD_ECHO       2 //get echo
#define CMD_INFO       3 //get information
#define CMD_SETADDR    4 //set net address
#define CMD_GETADDR    5 //get net address

//Specific SG-633 commands:

#define CMD_SETMODE    6 //set control mode
#define CMD_GETMODE    7 //get control mode
#define CMD_SETPAR     8 //set parameter value
#define CMD_GETPAR     9 //get parameter value
#define CMD_GETSELPAR 10 //get selected parameter value

//Unknown command:

#define CMD_UNKNOWN   11 //unknown command

#define MAX_CMD       11

//Commands names:

const SStr CmdNames[MAX_CMD + 1] =
  { "",
    "IO error: ",
    "Get echo error: ",
    "Get info error: ",
    "Set address error: ",
    "Get address error: ",
    "Set mode error: ",
    "Get mode error: ",
    "Set parameter error: ",
    "Get parameter error: ",
    "Get selected parameter error: ",
    "System error: " };

const SStr DevName = "SG-633\0"; //device name
const DWORD Baud = 38400;        //baud rate
const ULONG RxTo = 50;           //RX timeout, mS

//------------------------------ Variables: ---------------------------------

HANDLE DevHandle;            //device handle
CRITICAL_SECTION Cs;         //critical section
unsigned char TxData[FRAME]; //TX data buffer
unsigned char RxData[FRAME]; //RX data buffer
unsigned char addr, cmd, n;  //Wake protocol variables
char LastError[64] = "";     //last error string

//--------------------- Functions prototypes: -------------------------------

bool __fastcall CmdExe(unsigned char cmd, unsigned char n);
bool __fastcall ErrorReport(int cmd, int errc);

//---------------------------------------------------------------------------

#pragma argsused

int WINAPI DllEntryPoint(HINSTANCE hinst, unsigned long reason, void* lpReserved)
{
  switch(reason)
  {
    case DLL_PROCESS_ATTACH: InitializeCriticalSection(&Cs); break;
    case DLL_PROCESS_DETACH: DeleteCriticalSection(&Cs); break;
  }
  return 1;
}

//-------------------------- Execute command: -------------------------------

bool __fastcall CmdExe(unsigned char cmd, unsigned char n)
{
  setmem(RxData, FRAME, 0);                    //clear RX data
  if(DevHandle)                                //if port is opened
   {
     if(TxFrame(0, cmd, n, TxData) &&          //transmit frame
        RxFrame(RxTo, addr, cmd, n, RxData))   //receive frame
          return(ErrorReport(cmd, RxData[0])); //cuccess
     return(ErrorReport(CMD_ERR, ERR_NR));     //device is not respondong
   }
  return(ErrorReport(CMD_NOP, ERR_NO));        //port is closed, return
}

//-------------------------- Report of error: -------------------------------

bool __fastcall ErrorReport(int cmd, int errc)
{
  if(cmd == CMD_ECHO || cmd == CMD_INFO) errc = ERR_NO; //no error code
  if(errc == ERR_NO) cmd = CMD_NOP;  //if no error, empty command string
  if(cmd > MAX_CMD) cmd = MAX_CMD;   //check for command code out of range
  if(errc > MAX_ERR) errc = MAX_ERR; //check for error code out of range
  strcpy(LastError, CmdNames[cmd]);  //copy command name
  strcat(LastError, ErrorMsg[errc]); //copy error message
  return(errc == ERR_NO);
}

//---------------------------------------------------------------------------
//-------------------------- Exported functions -----------------------------
//---------------------------------------------------------------------------

//---------------------------- Open device: ---------------------------------

bool WINAPI SG633_OpenDevice(void)
{
  DWORD numDevs;
  LPCSTR DevInfo;
  if(!ListDevices(numDevs)) return(0);
  for(DWORD i = 0; i < numDevs; i++)
  {
    if(OpenPort(i, DevHandle))
      if(SetupPort(Baud))
        if(SG633_GetInfo(DevInfo))
          if(!strncmp(DevInfo, DevName, 6))
          {
            return(1);
          }
    if(DevHandle)
    {
      ClosePort(DevHandle);
    }
  }
  return(0);
}

//---------------------------- Close device: --------------------------------

bool WINAPI SG633_CloseDevice(void)
{
  return(ClosePort(DevHandle));
}

//------------------------- Get last error name: ----------------------------

void WINAPI SG633_GetLastError(LPCSTR &lpcStr)
{
  lpcStr = LastError;
}

//------------------------------ CMD_INFO: ----------------------------------

bool WINAPI SG633_GetInfo(LPCSTR &lpcStr)
{
  EnterCriticalSection(&Cs);
  cmd = (unsigned char) CMD_INFO;
  bool rx = CmdExe(cmd, 0);
  lpcStr = (LPCSTR)RxData;
  LeaveCriticalSection(&Cs);
  return(rx);
}

//----------------------------- CMD_SETMODE: --------------------------------

bool WINAPI SG633_SetMode(int mode)
{
  EnterCriticalSection(&Cs);
  cmd = (unsigned char) CMD_SETMODE;
  TxData[0] = (unsigned char) (mode & 0xFF);
  bool rx = CmdExe(cmd, 1);
  LeaveCriticalSection(&Cs);
  return(rx);
}

//----------------------------- CMD_GETMODE: --------------------------------

bool WINAPI SG633_GetMode(int &mode)
{
  EnterCriticalSection(&Cs);
  cmd = (unsigned char) CMD_GETMODE;
  bool rx = CmdExe(cmd, 0);
  mode = RxData[1];
  LeaveCriticalSection(&Cs);
  return(rx);
}

//----------------------------- CMD_SETPAR: ---------------------------------

bool WINAPI SG633_SetPar(int ch, int par, int val)
{
  EnterCriticalSection(&Cs);
  cmd = (unsigned char) CMD_SETPAR;
  TxData[0] = (unsigned char) (ch  & 0xFF);
  TxData[1] = (unsigned char) (par & 0xFF);
  TxData[2] = (unsigned char) ( val        & 0xFF);
  TxData[3] = (unsigned char) ((val >> 8)  & 0xFF);
  TxData[4] = (unsigned char) ((val >> 16) & 0xFF);
  TxData[5] = (unsigned char) ((val >> 24) & 0xFF);
  bool rx = CmdExe(cmd, 6);
  LeaveCriticalSection(&Cs);
  return(rx);
}

//----------------------------- CMD_GETPAR: ---------------------------------

bool WINAPI SG633_GetPar(int ch, int par, int &val)
{
  EnterCriticalSection(&Cs);
  cmd = (unsigned char) CMD_GETPAR;
  TxData[0] = (unsigned char) (ch  & 0xFF);
  TxData[1] = (unsigned char) (par & 0xFF);
  bool rx = CmdExe(cmd, 2);
  val = RxData[1] + (RxData[2] << 8) + (RxData[3] << 16) + (RxData[4] << 24);
  LeaveCriticalSection(&Cs);
  return(rx);
}

//---------------------------- CMD_GETSELPAR: -------------------------------

bool WINAPI SG633_GetSelPar(int &ch, int &par, int &val)
{
  EnterCriticalSection(&Cs);
  cmd = (unsigned char) CMD_GETSELPAR;
  bool rx = CmdExe(cmd, 0);
  ch  = RxData[1];
  par = RxData[2];
  val = RxData[3] + (RxData[4] << 8) + (RxData[5] << 16) + (RxData[6] << 24);
  LeaveCriticalSection(&Cs);
  return(rx);
}

//---------------------------------------------------------------------------


